==> bot4-0.0.8-py3-none-any.whl: bot4/bot/botbase.py <==
import logging, asyncio
from logging import Logger
from typing import Callable, Type, Union
from buffer import Buffer
from net.crypto import Cipher
from settings import Settings
from mcd import PacketsMcd, DefaultDataPacks
from router.packetsrouter import PacketRouter, PacketStorage
from router.baserouter import BaseRouter
from protocol.protocolbase import PortocolBase
from exceptions import ProtocolError, Protocols_LoopError


logging.basicConfig(encoding='utf-8')  #filename='log.log', filemode='w'


class BotBase:
    default_loglevel = logging.INFO
    compression_threshold = -1
    state = 0
    is_close = True
    packets_direction_d: str = 'download'
    packets_direction_s: str = 'upload'
    buff_type = Buffer
    mcd_type = PacketsMcd

    data_packets: DefaultDataPacks
    recv_buff: Buffer
    loop: asyncio.AbstractEventLoop
    transport: asyncio.Transport
    settings: Settings
    
    def __init__(self, settings: Settings, loop: asyncio.AbstractEventLoop = None):
        self.loop = asyncio.get_event_loop() if loop is None else loop
        self.logger = self._getlogger()

        self.settings = settings
        self.ip = settings.ip
        self.port = settings.port

        self.cipher = Cipher()
        self.recv_buff = self.buff_type()
        self.protocols = {}

        self.mcd = self.mcd_type(settings.version)
        self.data_packets = self.mcd.datapacks

        self.set_start()

    def set_start(self):
        pass

    @classmethod
    def _getlogger(cls):
        name = cls.__name__
        if name in Logger.manager.loggerDict:
            del Logger.manager.loggerDict[name]
        lg = logging.getLogger(name)
        lg.setLevel(cls.default_loglevel)
        return lg

    def add_protocol(self,
                     state: int,
                     state_datapackets: int = None,
                     router_packets_type: Type[PacketRouter] = PacketRouter,
                     *castom_routers: BaseRouter,
                     protocol_type: Type[PortocolBase] = None):
        if isinstance(state, int):
            try:
                if state_datapackets is None:
                    state_datapackets = state
                data_packets = self.data_packets[state_datapackets]
            except IndexError:
                raise Protocols_LoopError(f'state <{state}> not found')
        else:
            raise TypeError(f'state no valid type <{type(state)}>, valid types "int"')
        
        
        def _w(protocol_type: Type[PortocolBase]):
            if not issubclass(protocol_type, PortocolBase): raise ProtocolError('The protocol is not an example of a PortocolBase')

            router_packets = router_packets_type(self.loop, PacketStorage(data_packets.__getattribute__(self.packets_direction_d)))
            protocol = protocol_type(self, data_packets.__getattribute__(self.packets_direction_s), router_packets, state)

            self.protocols[state] = protocol
            if castom_routers:
                protocol.reg_routers(*castom_routers)

            return protocol_type
            
        if protocol_type is None:
            return _w
        else:
            return _w(protocol_type)

    def switch_protocol(self, state: int):
        try:
            new_prot = self.protocols[state]
            switch = getattr(self.transport.get_protocol(), 'switch', None)
            self.state = state
            if switch is not None: switch()
        
            self.transport.set_protocol(new_prot())
        except KeyError:
            raise ProtocolError(f'protocol state <{state}> not found')

    def switch_right(self):
        self.switch_protocol(self.state + 1)

    def switch_left(self):
        self.switch_protocol(self.state - 1)

    def _connection_lost(self, exc: Exception | None):
        self.is_close = True
        self.compression_threshold = -1
        self.recv_buff.clear()
        self.cipher.disable()
        if isinstance(exc, Exception):
            self.logger.error(exc_info=exc)
        self.logger.info('Client close')

    def close(self):
        if not self.is_close and not (getattr(self, 'transport', None) is None):
            self.transport.close()

    async def connect(self):
        if self.is_close:
            try:
                await self.loop.create_connection(self.protocols[0], host=self.ip, port=self.port)
                return True
            except Exception as e:
                self.logger.error(exc_info=e)
                return False

==> bot4-0.0.11-py3-none-any.whl: bot4/bot/botbase.py <==
from bot4.buffer import Buffer
from bot4.net.crypto import Cipher
from bot4.settings import Settings
from bot4.mcd import PacketsMcd, DefaultDataPacks
from bot4.router.packetsrouter import PacketRouter, PacketStorage
from bot4.router.baserouter import BaseRouter
from bot4.protocol.protocolbase import PortocolBase
from bot4.exceptions import ProtocolError, Protocols_LoopError
import logging, asyncio
from logging import Logger
from typing import Callable, Type, Union


logging.basicConfig(encoding='utf-8')  #filename='log.log', filemode='w'


class BotBase:
    default_loglevel = logging.INFO
    compression_threshold = -1
    state = 0
    is_close = True
    packets_direction_d: str = 'download'
    packets_direction_s: str = 'upload'
    buff_type = Buffer
    mcd_type = PacketsMcd

    data_packets: DefaultDataPacks
    recv_buff: Buffer
    loop: asyncio.AbstractEventLoop
    transport: asyncio.Transport
    settings: Settings
    
    def __init__(self, settings: Settings, loop: asyncio.AbstractEventLoop = None):
        self.loop = asyncio.get_event_loop() if loop is None else loop
        self.logger = self._getlogger()

        self.settings = settings
        self.ip = settings.ip
        self.port = settings.port

        self.cipher = Cipher()
        self.recv_buff = self.buff_type()
        self.protocols = {}

        self.mcd = self.mcd_type(settings.version)
        self.data_packets = self.mcd.datapacks

        self.set_start()

    def set_start(self):
        pass

    @classmethod
    def _getlogger(cls):
        name = cls.__name__
        if name in Logger.manager.loggerDict:
            del Logger.manager.loggerDict[name]
        lg = logging.getLogger(name)
        lg.setLevel(cls.default_loglevel)
        return lg

    def add_protocol(self,
                     state: int,
                     state_datapackets: int = None,
                     router_packets_type: Type[PacketRouter] = PacketRouter,
                     *castom_routers: BaseRouter,
                     protocol_type: Type[PortocolBase] = None):
        if isinstance(state, int):
            try:
                if state_datapackets is None:
                    state_datapackets = state
                data_packets = self.data_packets[state_datapackets]
            except IndexError:
                raise Protocols_LoopError(f'state <{state}> not found')
        else:
            raise TypeError(f'state no valid type <{type(state)}>, valid types "int"')
        
        
        def _w(protocol_type: Type[PortocolBase]):
            if not issubclass(protocol_type, PortocolBase): raise ProtocolError('The protocol is not an example of a PortocolBase')

            router_packets = router_packets_type(self.loop, PacketStorage(data_packets.__getattribute__(self.packets_direction_d)))
            protocol = protocol_type(self, data_packets.__getattribute__(self.packets_direction_s), router_packets, state)

            self.protocols[state] = protocol
            if castom_routers:
                protocol.reg_routers(*castom_routers)

            return protocol_type
            
        if protocol_type is None:
            return _w
        else:
            return _w(protocol_type)

    def switch_protocol(self, state: int):
        try:
            new_prot = self.protocols[state]
            switch = getattr(self.transport.get_protocol(), 'switch', None)
            self.state = state
            if switch is not None: switch()
        
            self.transport.set_protocol(new_prot())
        except KeyError:
            raise ProtocolError(f'protocol state <{state}> not found')

    def switch_right(self):
        self.switch_protocol(self.state + 1)

    def switch_left(self):
        self.switch_protocol(self.state - 1)

    def _connection_lost(self, exc: Exception | None):
        self.is_close = True
        self.compression_threshold = -1
        self.recv_buff.clear()
        self.cipher.disable()
        if isinstance(exc, Exception):
            self.logger.error(exc_info=exc)
        self.logger.info('Client close')

    def close(self):
        if not self.is_close and not (getattr(self, 'transport', None) is None):
            self.transport.close()

    async def connect(self):
        if self.is_close:
            try:
                await self.loop.create_connection(self.protocols[0], host=self.ip, port=self.port)
                return True
            except Exception as e:
                self.logger.error(exc_info=e)
                return False
